#include "CHAGApp.h"
#include <utils/Log.h>
#include <wx/propgrid/advprops.h>
#include <sstream>
#include <wx/event.h>

wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
	EVT_MENU(ID_ShowConsole, MyFrame::OnShowConsole)
	EVT_MENU(ID_ShowFullScreen, MyFrame::OnShowFullScreen)
	EVT_MENU(ID_ImportOBJ,	MyFrame::OnImportOBJ)
	EVT_MENU(ID_ReloadShaders, MyFrame::OnReloadShaders)
	EVT_MENU(wxID_EXIT,		MyFrame::OnExit)
	EVT_MENU(wxID_ABOUT,	MyFrame::OnAbout)
	EVT_MENU(wxID_SAVE,		MyFrame::OnSave)
	EVT_MENU(wxID_SAVEAS,	MyFrame::OnSaveAs)
	EVT_MENU(wxID_OPEN,		MyFrame::OnOpen)
wxEND_EVENT_TABLE()

bool CHAGApp::OnInit()
{
	m_scene = NULL; 
	m_fullScreen = false; 
	m_showConsole = true; 
	///////////////////////////////////////////////////////////////////////////
	// TODO: Very annoyingly, my application does not seem to want to honor
	//       the Close() call, for a long time. This is part of a quick fix 
	//		 that should be improved with whatever is the right way to do it. 
	///////////////////////////////////////////////////////////////////////////
	m_closeRequested = false; 

	///////////////////////////////////////////////////////////////////////////
	// Set up window
	///////////////////////////////////////////////////////////////////////////
	wxSize windowSize(applicationGLPaneWidth() + (applicationLoadsScene() ? 300 : 0), applicationGLPaneHeight() + 300); 
    m_frame = new MyFrame(applicationWindowText(), wxPoint(50,50), windowSize, this);
    m_h_sizer = new wxBoxSizer(wxHORIZONTAL);
	m_v_sizer_0 = new wxBoxSizer(wxVERTICAL);
	m_v_sizer_1 = new wxBoxSizer(wxVERTICAL);
	m_h_sizer->Add(m_v_sizer_0, 1, wxEXPAND | wxALL, 1); 
	m_h_sizer->Add(m_v_sizer_1, 3, wxEXPAND | wxALL, 1); 
	wxInitAllImageHandlers(); 

	///////////////////////////////////////////////////////////////////////////
	// The OpenGL Pane 
	///////////////////////////////////////////////////////////////////////////
	int args[] = {WX_GL_RGBA, WX_GL_DOUBLEBUFFER, WX_GL_DEPTH_SIZE, 16, 0};
    m_glPane = new wxGLPane(this, (wxFrame*) m_frame, args);
	m_glPane->SetInitialSize(wxSize(applicationGLPaneWidth(), applicationGLPaneHeight()));
    m_v_sizer_1->Add(m_glPane, 2, wxEXPAND, 1);

	if(applicationLoadsScene()) {
		///////////////////////////////////////////////////////////////////////////
		// Tree for showing scene
		///////////////////////////////////////////////////////////////////////////
		m_sceneTreeCtrl = new wxTreeCtrl(m_frame); 
		m_v_sizer_0->Add(m_sceneTreeCtrl, 3, wxEXPAND | wxALL, 1); 
		m_sceneTreeCtrl->Bind(wxEVT_TREE_SEL_CHANGED, &CHAGApp::OnSceneTreeCtrlSelectionChanged, this);
		m_sceneTreeCtrl->Bind(wxEVT_TREE_ITEM_MENU, &CHAGApp::OnSceneTreeCtrlMenu, this); 

		///////////////////////////////////////////////////////////////////////////
		// Property grid for viewing/modifying values of objects
		///////////////////////////////////////////////////////////////////////////
		m_sceneItemPropertyGrid = new wxPropertyGrid(m_frame, 3, wxDefaultPosition, wxSize(1,220), wxPG_SPLITTER_AUTO_CENTER | wxPG_DEFAULT_STYLE );
		m_v_sizer_0->Add(m_sceneItemPropertyGrid, 0, wxEXPAND | wxALL, 1); 
		m_sceneItemPropertyGrid->Bind(wxEVT_PG_CHANGED, &CHAGApp::OnScenePropertyGridChanged, this); 
	}

	///////////////////////////////////////////////////////////////////////////
	// A text control for warnings and information
	///////////////////////////////////////////////////////////////////////////
	m_console = new wxTextCtrl(m_frame, -1, "Standard out:\n", wxDefaultPosition, wxSize(100,60), wxTE_MULTILINE);
	m_v_sizer_1->Add(m_console, 1, wxEXPAND | wxALL, 1 );
	m_console->Show(m_showConsole); 

	///////////////////////////////////////////////////////////////////////////
	// This nifty thing steals cout
	///////////////////////////////////////////////////////////////////////////
	std::streambuf *sbOld = std::cout.rdbuf();
	std::cout.rdbuf(m_console);
    m_frame->SetSizer(m_h_sizer);
    m_frame->SetAutoLayout(true); 
	m_frame->Fit(); 
    m_frame->Show();

	///////////////////////////////////////////////////////////////////////////
	// Get file to open from LRU dialog
	///////////////////////////////////////////////////////////////////////////
	if(applicationLoadsScene()) {
		m_LRUFilenameManager = new LRUFilenameManager("LRUFilenames.txt"); 
		std::vector<std::string> filenames = m_LRUFilenameManager->getFilenames(); 
		if(filenames.size() > 0) {
			wxArrayString choices; 
			for(int i=0; i<filenames.size(); i++) choices.Add(filenames[i]); 
			wxSingleChoiceDialog *dlg = new wxSingleChoiceDialog(m_frame, "Choose a recently used file", "Choose File", choices); 
			if(dlg->ShowModal() == wxID_OK) {
				LOG_INFO("LRU File chosen: " << filenames[dlg->GetSelection()]);
				m_initialFilename = filenames[dlg->GetSelection()]; 
			}
		}
		if(m_initialFilename == "") {
			wxFileDialog openFileDialog(m_frame, _("Open SCENE file"), "", "",
										"SCENE files (*.scene)|*.scene", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
			if(openFileDialog.ShowModal() == wxID_OK) 
				m_initialFilename = openFileDialog.GetPath(); 			
		}

		m_scene = new Scene; 
		if(m_initialFilename == "") {
			LOG_INFO("No initial file chosen.");
		}
		else {
			load(m_initialFilename); 
			m_LRUFilenameManager->addFilename(m_initialFilename); 
			m_LRUFilenameManager->writeToFile(); 
		}
		initializeSceneTreeCtrl(); 
	}



	return true;
}

void CHAGApp::initializeSceneTreeCtrl()
{
	m_sceneTreeCtrl->DeleteAllItems(); 
	m_sceneItems.clear(); 
	m_sceneMaterials.clear(); 
	wxTreeItemId rootID = m_sceneTreeCtrl->AddRoot("Scene"); 
	m_lightRootID = m_sceneTreeCtrl->AppendItem(rootID, "Lights"); 
	for(auto l : m_scene->m_lights) {
		wxTreeItemId lightID = m_sceneTreeCtrl->AppendItem(m_lightRootID, l->m_name); 
		m_sceneItems[lightID] = l; 
	}
	m_shadowGeneratorRootID = m_sceneTreeCtrl->AppendItem(rootID, "Shadow Generators"); 
	for(auto s : m_scene->m_shadowGenerators) {
		wxTreeItemId shadowCasterID = m_sceneTreeCtrl->AppendItem(m_shadowGeneratorRootID, s->m_name); 
		m_sceneItems[shadowCasterID] = s; 
	}
	m_cameraRootID = m_sceneTreeCtrl->AppendItem(rootID, "Cameras"); 
	for(auto c : m_scene->m_cameras) {
		wxTreeItemId cameraID = m_sceneTreeCtrl->AppendItem(m_cameraRootID, c->m_name); 
		m_sceneItems[cameraID] = c; 
	}
	m_modelRootID = m_sceneTreeCtrl->AppendItem(rootID, "Models"); 
	for(auto m : m_scene->m_models) {
		wxTreeItemId modelID = m_sceneTreeCtrl->AppendItem(m_modelRootID, m->m_name); 
		m_sceneItems[modelID] = m;

		for(auto mat : m->m_materials) {
			wxTreeItemId materialID = m_sceneTreeCtrl->AppendItem(modelID, mat.first); 
			m_sceneMaterials[materialID] = mat.second; 
		}
	}
	m_sceneTreeCtrl->ExpandAll(); 
}

void CHAGApp::OnSceneTreeCtrlSelectionChanged(wxTreeEvent& event) 
{
	wxTreeItemId selected = m_sceneTreeCtrl->GetSelection();
	Scene::Item * item = m_sceneItems.count(selected) > 0 ? m_sceneItems[selected] : NULL; 
	Scene::Material * material = m_sceneMaterials.count(selected) > 0 ? m_sceneMaterials[selected] : NULL; 
	if(item) {
		Scene::Light * light = dynamic_cast<Scene::Light *>(item); 
		Scene::Camera * camera = dynamic_cast<Scene::Camera *>(item); 
		Scene::Model * model = dynamic_cast<Scene::Model *>(item); 
		Scene::ShadowGenerator * shadowGenerator = dynamic_cast<Scene::ShadowGenerator *>(item); 
	
		m_sceneItemPropertyGrid->Clear(); 
			
		if(!shadowGenerator) {
			m_sceneItemPropertyGrid->Append(new wxPropertyCategory("Transform")); 
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Position X", wxPG_LABEL, item->m_transform.pos.x));
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Position Y", wxPG_LABEL, item->m_transform.pos.y));
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Position Z", wxPG_LABEL, item->m_transform.pos.z));
		}
		else {
			m_sceneItemPropertyGrid->Append(new wxPropertyCategory("Transform")); 
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Position X", wxPG_LABEL, shadowGenerator->m_light->m_transform.pos.x));
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Position Y", wxPG_LABEL, shadowGenerator->m_light->m_transform.pos.y));
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Position Z", wxPG_LABEL, shadowGenerator->m_light->m_transform.pos.z));
		}

		if(light) {
			LOG_INFO("Light Selected"); 
			m_sceneItemPropertyGrid->Append(new wxPropertyCategory("Emittance")); 
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Radiance", wxPG_LABEL, light->m_radiance));
			m_sceneItemPropertyGrid->Append(new wxColourProperty("Color", wxPG_LABEL, toWxColour(light->m_color)));
			Scene::DiscDiffuseLight * ddl = dynamic_cast<Scene::DiscDiffuseLight *>(item); 
			if(ddl) {
				m_sceneItemPropertyGrid->Append(new wxPropertyCategory("Disc Diffuse Light")); 
				m_sceneItemPropertyGrid->Append(new wxFloatProperty("Radius", wxPG_LABEL, ddl->m_radius));
			}
		}
		if(camera) {
			LOG_INFO("Camera Selected");
			m_sceneItemPropertyGrid->Append(new wxPropertyCategory("Projection")); 
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Near plane", wxPG_LABEL, camera->m_near));
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Far plane", wxPG_LABEL, camera->m_far));
			
			Scene::PerspectiveCamera *perspective_camera = dynamic_cast<Scene::PerspectiveCamera *>(camera); 
			if(perspective_camera) {
				LOG_INFO("Perspective Camera Selected"); 
				m_sceneItemPropertyGrid->Append(new wxFloatProperty("FOV", wxPG_LABEL, perspective_camera->m_fov));
			}
			else {
				LOG_INFO("Ortho Camera Selected");
			}
		}
		if(model) {
			LOG_INFO("Model Selected"); 
		}
		if(shadowGenerator) {
			Scene::ShadowMapShadowGenerator *shadow_map_shadow_generator = dynamic_cast<Scene::ShadowMapShadowGenerator *>(shadowGenerator); 
			m_sceneItemPropertyGrid->Clear(); 
			if(shadow_map_shadow_generator) {
				LOG_INFO("Shadow Map Selected");
				m_sceneItemPropertyGrid->Append(new wxPropertyCategory("Shadow Map")); 
				m_sceneItemPropertyGrid->Append(new wxFloatProperty("Near plane", wxPG_LABEL, shadow_map_shadow_generator->m_near));
				m_sceneItemPropertyGrid->Append(new wxFloatProperty("Far plane", wxPG_LABEL, shadow_map_shadow_generator->m_far));
				m_sceneItemPropertyGrid->Append(new wxFloatProperty("Aspect Ratio", wxPG_LABEL, shadow_map_shadow_generator->m_aspect_ratio));
				m_sceneItemPropertyGrid->Append(new wxFloatProperty("FOV", wxPG_LABEL, shadow_map_shadow_generator->m_fov));
				m_sceneItemPropertyGrid->Append(new wxIntProperty("Width", wxPG_LABEL, shadow_map_shadow_generator->m_resolution.x));
				m_sceneItemPropertyGrid->Append(new wxIntProperty("Height", wxPG_LABEL, shadow_map_shadow_generator->m_resolution.y));
				m_sceneItemPropertyGrid->Append(new wxFloatProperty("Polygon Offset Factor", wxPG_LABEL, shadow_map_shadow_generator->m_polygon_offset_factor));
				m_sceneItemPropertyGrid->Append(new wxFloatProperty("Polygon Offset Units", wxPG_LABEL, shadow_map_shadow_generator->m_polygon_offset_units));
			}
		}
	}
	else if(material) 
	{
			LOG_INFO("material " << material->m_name << " selected"); 
			m_sceneItemPropertyGrid->Clear(); 
			m_sceneItemPropertyGrid->Append(new wxPropertyCategory(	"Reflective Coating")); 
			m_sceneItemPropertyGrid->Append(new wxBoolProperty("RC Enable", wxPG_LABEL, material->m_reflective_coating_enable)); 
			m_sceneItemPropertyGrid->Append(new wxColourProperty("RC Reflectance", wxPG_LABEL, toWxColour(material->m_reflective_coating_reflectance))); 
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("RC Roughness", wxPG_LABEL, material->m_reflective_coating_roughness));
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("RC 0 degrees", wxPG_LABEL, material->m_reflective_coating_0_degrees));
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("RC 90 degrees", wxPG_LABEL, material->m_reflective_coating_90_degrees));
			m_sceneItemPropertyGrid->Append(new wxPropertyCategory(	"Diffuse")); 
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Diffuse Weight", wxPG_LABEL, material->m_diffuse_weight));
			m_sceneItemPropertyGrid->Append(new wxColourProperty("Diffuse Reflectance", wxPG_LABEL, toWxColour(material->m_diffuse_reflectance))); 
			m_sceneItemPropertyGrid->Append(new wxPropertyCategory("Glossy")); 
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Glossy Weight", wxPG_LABEL, material->m_glossy_weight));
			m_sceneItemPropertyGrid->Append(new wxColourProperty("Glossy Reflectance", wxPG_LABEL, toWxColour(material->m_glossy_reflectance))); 
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Glossy Roughness", wxPG_LABEL, material->m_glossy_roughness));
	}
	else if(selected == m_sceneTreeCtrl->GetRootItem()) 
	{
		LOG_INFO("Scene selected"); 
		m_sceneItemPropertyGrid->Clear(); 
		m_sceneItemPropertyGrid->Append(new wxPropertyCategory(	"Scene")); 
		m_sceneItemPropertyGrid->Append(new wxFloatProperty("Unit Multiplier", wxPG_LABEL, m_scene->m_sceneUnitMultiplier));
		m_sceneItemPropertyGrid->Append(new wxDirProperty("Environment Map", wxPG_LABEL)); 
	}
	else {
		m_sceneItemPropertyGrid->Clear(); 
		LOG_INFO("Selected item was not found"); 
	}
}

void CHAGApp::OnScenePropertyGridChanged(wxPropertyGridEvent &event)
{
	wxTreeItemId selected = m_sceneTreeCtrl->GetSelection();
	Scene::Item * item = m_sceneItems[selected]; 
	Scene::Material * material = m_sceneMaterials[selected]; 
	wxAny value = event.GetValue(); 		
	if(item) {
		Scene::Light * light = dynamic_cast<Scene::Light *>(item); 
		Scene::Camera * camera = dynamic_cast<Scene::Camera *>(item); 
		Scene::Model * model = dynamic_cast<Scene::Model *>(item); 
		Scene::ShadowGenerator * shadowGenerator = dynamic_cast<Scene::ShadowGenerator *>(item); 

		if(!shadowGenerator) {
			if(event.GetPropertyName() == "Position X") item->m_transform.pos.x = value.As<float>(); 
			if(event.GetPropertyName() == "Position Y") item->m_transform.pos.y = value.As<float>(); 
			if(event.GetPropertyName() == "Position Z") item->m_transform.pos.z = value.As<float>(); 
		}
		else {
			if(event.GetPropertyName() == "Position X") shadowGenerator->m_light->m_transform.pos.x = value.As<float>(); 
			if(event.GetPropertyName() == "Position Y") shadowGenerator->m_light->m_transform.pos.y = value.As<float>(); 
			if(event.GetPropertyName() == "Position Z") shadowGenerator->m_light->m_transform.pos.z = value.As<float>(); 
		}

		if(light) {
			if(event.GetPropertyName() == "Radiance") light->m_radiance = value.As<float>(); 
			if(event.GetPropertyName() == "Color") light->m_color = toFloat3Color(value.As<wxColour>()); 
			LOG_INFO("Light Property Changed"); 
		}
		if(camera) {
			LOG_INFO("Camera Property Changed"); 
			if(event.GetPropertyName() == "Near plane") camera->m_near = value.As<float>(); 
			if(event.GetPropertyName() == "Far plane") camera->m_far = value.As<float>(); 
			Scene::PerspectiveCamera *perspective_camera = dynamic_cast<Scene::PerspectiveCamera *>(camera); 
			if(perspective_camera) {
				LOG_INFO("Perspective Camera Property Changed"); 
				if(event.GetPropertyName() == "FOV") perspective_camera->m_fov = value.As<float>(); 
			}
		}
		if(model) {
			LOG_INFO("Model Property Changed"); 
		}
		Scene::DiscDiffuseLight * ddl = dynamic_cast<Scene::DiscDiffuseLight *>(item); 
		if(ddl) {
			if(event.GetPropertyName() == "Radius") ddl->m_radius = value.As<float>(); 
		}
		if(shadowGenerator) {
			LOG_INFO("Shadow Caster Property Changed"); 
			Scene::ShadowMapShadowGenerator *shadow_map_shadow_generator = dynamic_cast<Scene::ShadowMapShadowGenerator *>(shadowGenerator); 
			if(shadow_map_shadow_generator) {
				if(event.GetPropertyName() == "Near plane") shadow_map_shadow_generator->m_near = value.As<float>(); 
				if(event.GetPropertyName() == "Far plane") shadow_map_shadow_generator->m_far = value.As<float>(); 
				if(event.GetPropertyName() == "FOV") shadow_map_shadow_generator->m_fov = value.As<float>(); 
				if(event.GetPropertyName() == "Aspect Ratio") shadow_map_shadow_generator->m_aspect_ratio = value.As<float>(); 
				if(event.GetPropertyName() == "Width") { 
					shadow_map_shadow_generator->m_resolution.x = value.As<int>(); 
					shadow_map_shadow_generator->resize(shadow_map_shadow_generator->m_resolution.x, shadow_map_shadow_generator->m_resolution.y); 
				}
				if(event.GetPropertyName() == "Height") { 
					shadow_map_shadow_generator->m_resolution.y = value.As<int>(); 
					shadow_map_shadow_generator->resize(shadow_map_shadow_generator->m_resolution.x, shadow_map_shadow_generator->m_resolution.y); 
				}
				if(event.GetPropertyName() == "Polygon Offset Factor") shadow_map_shadow_generator->m_polygon_offset_factor = value.As<float>(); 
				if(event.GetPropertyName() == "Polygon Offset Units") shadow_map_shadow_generator->m_polygon_offset_units = value.As<float>(); 
			}
		}
	}
	else if(material) 
	{
		wxTreeItemId parentModelId = m_sceneTreeCtrl->GetItemParent(selected); 
		Scene::Model *model = m_scene->getModel(std::string(m_sceneTreeCtrl->GetItemText(parentModelId).mb_str())); 
		std::string materialName = std::string(m_sceneTreeCtrl->GetItemText(selected).mb_str()); 
		Scene::Material *material = model->m_materials[materialName]; 

		if(event.GetPropertyName() == "RC Enable") { material->m_reflective_coating_enable = value.As<bool>(); }
		if(event.GetPropertyName() == "RC Reflectance") { material->m_reflective_coating_reflectance = toFloat3Color(value.As<wxColour>()); }
		if(event.GetPropertyName() == "RC Roughness") { material->m_reflective_coating_roughness = value.As<float>(); }
		if(event.GetPropertyName() == "RC 0 degrees") { material->m_reflective_coating_0_degrees = value.As<float>(); }
		if(event.GetPropertyName() == "RC 90 degrees") { material->m_reflective_coating_90_degrees = value.As<float>(); }
		if(event.GetPropertyName() == "Diffuse Weight") { material->m_diffuse_weight = value.As<float>(); }
		if(event.GetPropertyName() == "Diffuse Reflectance") { material->m_diffuse_reflectance = toFloat3Color(value.As<wxColour>()); }
		if(event.GetPropertyName() == "Glossy Weight") { material->m_glossy_weight = value.As<float>(); }
		if(event.GetPropertyName() == "Glossy Reflectance") { material->m_glossy_reflectance = toFloat3Color(value.As<wxColour>()); }
		if(event.GetPropertyName() == "Glossy Roughness") { material->m_glossy_roughness = value.As<float>(); }

		LOG_INFO("Material Property '" << event.GetPropertyName() << "'Changed"); 
	}
	else if(selected == m_sceneTreeCtrl->GetRootItem()) {
		if(event.GetPropertyName() == "Unit Multiplier") { m_scene->m_sceneUnitMultiplier = value.As<float>(); }
		if(event.GetPropertyName() == "Environment Map") {

			wxImage posx(value.As<wxString>() + wxString("/posx.jpg")); 
			wxImage negx(value.As<wxString>() + wxString("/negx.jpg")); 
			wxImage posy(value.As<wxString>() + wxString("/posy.jpg")); 
			wxImage negy(value.As<wxString>() + wxString("/negy.jpg")); 
			wxImage posz(value.As<wxString>() + wxString("/posz.jpg")); 
			wxImage negz(value.As<wxString>() + wxString("/negz.jpg")); 
			
			m_scene->m_environmentMap.load(	posx.GetData(), posy.GetData(), posz.GetData(), 
											negx.GetData(), negy.GetData(), negz.GetData(), 
											posx.GetSize().x, posx.GetSize().y); 
		}
	}
}

void CHAGApp::OnSceneTreeCtrlMenu(wxTreeEvent& event) 
{
	wxTreeItemId selected = event.GetItem(); 
	Scene::DiscDiffuseLight * light = dynamic_cast<Scene::DiscDiffuseLight *>(m_sceneItems[selected]); 
	Scene::PerspectiveCamera * camera = dynamic_cast<Scene::PerspectiveCamera *>(m_sceneItems[selected]); 
	Scene::Model * model = dynamic_cast<Scene::Model *>(m_sceneItems[selected]); 
	Scene::ShadowMapShadowGenerator *shadowGenerator = dynamic_cast<Scene::ShadowMapShadowGenerator *>(m_sceneItems[selected]); 
	Scene::Item * item = dynamic_cast<Scene::Item *>(m_sceneItems[selected]); 
	if(item) {
		wxMenu menu; 
		enum { RENAME, CLONE, REMOVE, CONTROL, VIEW, VIEW_AND_CONTROL, CREATE_SHADOWMAP }; 
		menu.Append(RENAME, wxT("Rename")); 
		if(!shadowGenerator)menu.Append(CLONE, wxT("Clone")); 		
		menu.Append(REMOVE, wxT("Remove")); 
		menu.Append(VIEW, "View"); 
		if(!shadowGenerator) menu.Append(CONTROL, "Control"); 
		if(!shadowGenerator) menu.Append(VIEW_AND_CONTROL, "View and Control"); 
		if(light) menu.Append(CREATE_SHADOWMAP, "Create Shadowmap..."); 
		switch(m_sceneTreeCtrl->GetPopupMenuSelectionFromUser(menu)) 
		{
		case RENAME:
			{
				wxTextEntryDialog dlg(m_frame, "Enter new name", "Rename", item->m_name); 
				if(dlg.ShowModal() == wxID_OK) {
					item->rename(std::string(dlg.GetValue().mb_str())); 
					m_sceneTreeCtrl->SetItemText(selected, item->m_name); 
				}
			}
			break; 
		case CLONE:
			{
				std::string name_of_new = item->m_name;
				while(name_of_new == item->m_name) {
					wxTextEntryDialog dlg(m_frame, "Enter name for new item", "Clone", item->m_name); 
					if(dlg.ShowModal() == wxID_OK) {
						name_of_new = dlg.GetValue(); 
						m_sceneTreeCtrl->SetItemText(selected, name_of_new); 
					}
				}
				Scene::Item * new_item = item->clone(); 
				new_item->m_name = name_of_new; 
				if(camera) m_scene->m_cameras.push_back((Scene::Camera *)new_item);  
				if(light) m_scene->m_lights.push_back((Scene::Light *)new_item); 
				if(model) m_scene->m_models.push_back((Scene::Model *)new_item); 
				initializeSceneTreeCtrl(); 
			}
			break; 
		case REMOVE:
			{
				if(camera) {
					std::vector<Scene::Camera *>::iterator position = std::find(m_scene->m_cameras.begin(), m_scene->m_cameras.end(), camera);
					m_scene->m_cameras.erase(position); 
					delete camera; 
				}
				if(light) {
					std::vector<Scene::Light *>::iterator position = std::find(m_scene->m_lights.begin(), m_scene->m_lights.end(), light);
					m_scene->m_lights.erase(position); 
					delete light; 
				}
				if(model) {
					std::vector<Scene::Model *>::iterator position = std::find(m_scene->m_models.begin(), m_scene->m_models.end(), model);
					m_scene->m_models.erase(position); 
					delete model; 
				}
				if(shadowGenerator) {
					std::vector<Scene::ShadowGenerator *>::iterator position = std::find(m_scene->m_shadowGenerators.begin(), m_scene->m_shadowGenerators.end(), shadowGenerator);
					m_scene->m_shadowGenerators.erase(position); 
					delete shadowGenerator; 
				}
				initializeSceneTreeCtrl(); 
			}
			break; 
		case VIEW: 
			m_scene->setCurrentRenderableItem(item->m_name);
			break;
		case CONTROL: 
			m_scene->setCurrentControllableItem(item->m_name);
			break;
		case VIEW_AND_CONTROL: 
			m_scene->setCurrentControllableItem(item->m_name);
			m_scene->setCurrentRenderableItem(item->m_name);
			break;
		case CREATE_SHADOWMAP: 
			{
				Scene::ShadowMapShadowGenerator *new_item = new Scene::ShadowMapShadowGenerator; 
				new_item->m_light = light; 
				new_item->m_aspect_ratio = 1.0f; 
				new_item->m_fov = 60.0f; 
				new_item->m_near = 0.1f; 
				new_item->m_far = 1000.0f; 
				new_item->resize(512, 512);
				new_item->m_polygon_offset_factor = 1.0f; 
				new_item->m_polygon_offset_units = 1.0f; 
				std::stringstream ss; 
				ss << "ShadowMap:" << light->m_name; 
				new_item->m_name = ss.str(); 
				m_scene->m_shadowGenerators.push_back((Scene::ShadowGenerator *)new_item); 
				initializeSceneTreeCtrl(); 
				break;
			}
		}
	}
	if(model) {
	}
	Scene::Material * material = m_sceneMaterials[selected]; 
	if(material) {
	}
}

void CHAGApp::firstFrame()
{
	///////////////////////////////////////////////////////////////////////////
	// Initialize GLEW. Must be done after GL context is created and before
	// anything glewish is done. 
	///////////////////////////////////////////////////////////////////////////
	glewInit(); 

	///////////////////////////////////////////////////////////////////////////
	// Application dependent post GL initialization
	///////////////////////////////////////////////////////////////////////////
	onFirstFrame(); 
}

void CHAGApp::save(std::string filename) 
{
	if(m_scene != NULL) { // Well, why wouldn't it? 
		if(!m_scene->save(filename)) {
			wxMessageBox("Save failed", "ERROR"); 
		}; 
		if(filename != "") {
			m_LRUFilenameManager->addFilename(filename); 
			m_LRUFilenameManager->writeToFile(); 
		}
	}
}

void CHAGApp::load(std::string filename) 
{
	if(!m_scene->load(filename, this)) {
		wxMessageBox("Load failed", "ERROR"); 
	}; 
	m_LRUFilenameManager->addFilename(filename); 
	m_LRUFilenameManager->writeToFile(); 



	initializeSceneTreeCtrl(); 
	onSceneChanged(); 
}

void CHAGApp::importOBJ(std::string filename)
{
	wxFileName f; 
	f.Assign(filename); 
	f.MakeRelativeTo("."); 
	filename = f.GetFullPath(); 
	if(!m_scene->importOBJ(filename)) {
		wxMessageBox("OBJ import failed", "ERROR"); 
	}
	initializeSceneTreeCtrl(); 
	onSceneChanged(); 
}

MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size, CHAGApp *chagApp)
	: wxFrame(NULL, wxID_ANY, title, pos, size)
{
	m_CHAGApp = chagApp; 
	wxMenu *menuFile = new wxMenu;
	if(m_CHAGApp->applicationLoadsScene()) {
		menuFile->Append(wxID_OPEN, "&Open File...\tCtrl-O"); 
		menuFile->Append(wxID_SAVE, "&Save\tCtrl-S"); 
		menuFile->Append(wxID_SAVEAS, "&Save as...\tCtrl-Shift-S"); 
		menuFile->Append(ID_ImportOBJ, "&Import OBJ...\tCtrl-H");
		menuFile->AppendSeparator();
	}
	menuFile->Append(ID_ReloadShaders, "&Reload Shaders\tShift-R");
	menuFile->Append(ID_ShowFullScreen, "&Full Screen\tAlt-Enter");
	menuFile->Append(ID_ShowConsole, "&Console\tAlt-C");
	menuFile->Append(wxID_EXIT, "&Exit...\tCtrl+C");
	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append( menuFile, "&File" );

	Bind(wxEVT_MENU, &MyFrame::OnMenuSelection, this); 
	Bind(wxEVT_MENU, &MyFrame::OnExit, this, wxID_EXIT); 
	Bind(wxEVT_MENU, &MyFrame::OnSave, this, wxID_SAVE); 
	Bind(wxEVT_MENU, &MyFrame::OnSaveAs, this, wxID_SAVEAS); 
	Bind(wxEVT_MENU, &MyFrame::OnOpen, this, wxID_OPEN); 
	Bind(wxEVT_MENU, &MyFrame::OnImportOBJ, this, ID_ImportOBJ); 
	Bind(wxEVT_MENU, &MyFrame::OnReloadShaders, this, ID_ReloadShaders); 
	Bind(wxEVT_MENU, &MyFrame::OnShowFullScreen, this, ID_ShowFullScreen); 
	Bind(wxEVT_MENU, &MyFrame::OnShowConsole, this, ID_ShowConsole); 

	m_CHAGApp->onApplicationCreateMenuBar(menuBar); 

	SetMenuBar( menuBar );
	CreateStatusBar();
	SetStatusText( "Welcome to wxWidgets!" );
}

void MyFrame::OnMenuSelection(wxCommandEvent &event)
{
	LOG_INFO("Unhandled Menu Command. Calling application.");
	m_CHAGApp->onUnhandledMenuCommand(event); 
}

void MyFrame::OnSaveAs(wxCommandEvent &event)
{
    wxFileDialog saveFileDialog(this, _("Save SCENE file"), "", "",
                       "SCENE files (*.scene)|*.scene", wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(saveFileDialog.ShowModal() == wxID_CANCEL) return;
	std::string filename = saveFileDialog.GetPath(); 
	LOG_INFO("Save as..." << filename);
	m_CHAGApp->save(filename);
}

void MyFrame::OnSave(wxCommandEvent &event)
{
	LOG_INFO("Save");
	m_CHAGApp->save(""); 
}

void MyFrame::OnOpen(wxCommandEvent &event)
{
    wxFileDialog openFileDialog(this, _("Open SCENE file"), "", "",
                       "SCENE files (*.scene)|*.scene", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
	if(openFileDialog.ShowModal() == wxID_CANCEL) return;
	std::string filename = openFileDialog.GetPath(); 
	LOG_INFO("Load " << filename);
	m_CHAGApp->load(filename);
}

void MyFrame::OnExit(wxCommandEvent& event)
{
	Close( true );
	LOG_INFO("Attempted to close");
	m_CHAGApp->m_closeRequested = true; 
}
void MyFrame::OnAbout(wxCommandEvent& event)
{
	wxMessageBox( "This is a wxWidgets' Hello world sample",
		"About Hello World", wxOK | wxICON_INFORMATION );
}
void MyFrame::OnImportOBJ(wxCommandEvent& event)
{
    wxFileDialog openFileDialog(this, _("Open OBJ file"), "", "",
                       "OBJ files (*.obj)|*.obj", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
	if(openFileDialog.ShowModal() == wxID_CANCEL) return;
	std::string filename = openFileDialog.GetPath(); 
	LOG_INFO("Import OBJ " << filename);
	m_CHAGApp->importOBJ(filename);
}

void MyFrame::OnReloadShaders(wxCommandEvent &event)
{
	m_CHAGApp->onReloadShaders(); 
}

void MyFrame::OnShowFullScreen(wxCommandEvent &event) { m_CHAGApp->OnShowFullScreen(); }
void MyFrame::OnShowConsole(wxCommandEvent &event) { m_CHAGApp->OnShowConsole(); }