#pragma once
#include <wx/wxprec.h>
#include "wxGLPane.h"
#include <iostream>
#include <wx/propgrid/propgrid.h>
#include <wx/treectrl.h>
#include <wx/splitter.h>
#include <scene/Scene.h>
#include <map>
#include <linmath/linmath.h> 
#include "LRUFilenameManager.h"

class CHAGApp; 

inline wxColour toWxColour(const chag::float3 &color) { return wxColour(color.x * 255, color.y * 255, color.z * 255); }
inline chag::float3 toFloat3Color(const wxColour &color) { return chag::make_vector(color.Red()/255.0f, color.Green()/255.0f, color.Blue()/255.0f); }

class MyFrame: public wxFrame
{
public:
	MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size, CHAGApp *chagApp);
private:
	CHAGApp *m_CHAGApp; 
	void OnImportOBJ(wxCommandEvent& event);
	void OnReloadShaders(wxCommandEvent &event); 
	void OnExit(wxCommandEvent& event);
	void OnAbout(wxCommandEvent& event);
	void OnSave(wxCommandEvent &event); 
	void OnSaveAs(wxCommandEvent &event); 
	void OnOpen(wxCommandEvent &event); 
	void OnMenuSelection(wxCommandEvent &event); 
	void OnShowFullScreen(wxCommandEvent &event);
	void OnShowConsole(wxCommandEvent &event);
	wxDECLARE_EVENT_TABLE();
};

class CHAGApp :	public wxApp, public SceneExtender
{
protected: 
	MyFrame *m_frame; 
	wxGLPane *m_glPane;

	wxTreeCtrl *m_sceneTreeCtrl; 
	std::map<wxTreeItemId, Scene::Item *> m_sceneItems; 
	std::map<wxTreeItemId, Scene::Material *> m_sceneMaterials; 

	wxTreeItemId m_lightRootID, m_cameraRootID, m_modelRootID, m_shadowGeneratorRootID; 

	virtual void initializeSceneTreeCtrl(); 
	virtual void OnSceneTreeCtrlSelectionChanged(wxTreeEvent &event); 
	virtual void OnSceneTreeCtrlMenu(wxTreeEvent &event); 
	virtual void OnScenePropertyGridChanged(wxPropertyGridEvent &event); 
public: 
	virtual void onApplicationCreateMenuBar(wxMenuBar *menuBar) {}; 
	virtual void onUnhandledMenuCommand(wxCommandEvent &event) {}; 

	wxPropertyGrid *m_sceneItemPropertyGrid; 

public:
	int m_gl_pane_width, m_gl_pane_height; 

	std::string m_initialFilename; 
	LRUFilenameManager *m_LRUFilenameManager; 
	Scene *m_scene; 
	virtual bool handleUnknownType(std::string &type, std::istream &is, Scene *scene) { LOG_INFO("No scene extender for: " << type); return false; }; 
	void save(std::string filename); 
	void load(std::string filename); 
	void importOBJ(std::string filename); 

	virtual bool OnInit();

	///////////////////////////////////////////////////////////////////////////
	// TODO: Very annoyingly, my application does not seem to want to honor
	//       the Close() call, for a long time. This is part of a quick fix 
	//		 that should be improved with whatever is the right way to do it. 
	///////////////////////////////////////////////////////////////////////////
	bool m_closeRequested; 

	void firstFrame(); 
	virtual void onFirstFrame() {};
	virtual void onDisplay() = 0; 
	virtual void onReshape(int w, int h) {}; 
	virtual void onSceneChanged() {}; 
	virtual void onViewChanged() {}; 
	virtual void onReloadShaders() {}; 
	virtual void onMouse(int x, int y, bool leftIsDown, bool middleIsDown, bool rightIsDown) {}; 
	virtual bool applicationLoadsScene() { return true; };
	virtual std::string applicationWindowText() { return std::string("WXWidgets Application"); }; 
	virtual int applicationGLPaneWidth() { return 1280; }; 
	virtual int applicationGLPaneHeight() { return 720; }; 

	wxTextCtrl *m_console; 

	bool m_fullScreen; 
	wxBoxSizer *m_h_sizer; 
	wxBoxSizer *m_v_sizer_0; 
	wxBoxSizer *m_v_sizer_1; 
	void refit() {
		//m_frame->Fit();
		//m_frame->SetSizerAndFit(m_v_sizer_1);
		//m_frame->SetSizerAndFit(m_h_sizer);
	}
	void OnShowFullScreen() { 
		m_fullScreen = !m_fullScreen; 
		m_glPane->SetSize(300, 300); 
		m_frame->ShowFullScreen(m_fullScreen); 
		refit(); 
	}
	bool m_showConsole; 
	void OnShowConsole() {
		m_showConsole = !m_showConsole; 
		m_console->Show(m_showConsole); 
		m_h_sizer->Fit(m_frame);  
		m_v_sizer_0->Fit(m_frame);  
		m_v_sizer_1->Fit(m_frame);  
	}
};	

enum
{
	ID_ImportOBJ = 1,
	ID_ReloadShaders = 2,
	ID_ShowFullScreen = 3,
	ID_ShowConsole = 4
};
