#include "wxGLPane.h"
#include "CHAGApp.h"
#include <iostream>
#include <utils/Log.h>
#include <linmath/streamoperators.h>

using namespace std; 

wxGLPane::wxGLPane(CHAGApp *chagApp, wxFrame* parent, int* args) :
	wxGLCanvas(parent, wxID_ANY, args, wxDefaultPosition, wxDefaultSize, wxFULL_REPAINT_ON_RESIZE) ,
	m_chagApp(chagApp)
{
	m_context = new wxGLContext(this);
	m_firstFrame = true; 
	m_display_timer = new wxTimer(this, 0); 
	m_display_timer->Start(32); 
	m_mouseSensitivity = 1.0f; 
	m_forwardVelocity = 0.0f; 
	m_strafeVelocity = 0.0f; 
	m_ascentVelocity = 0.0f; 
	m_frame_finished = true; 
	// To avoid flashing on MSW
	SetBackgroundStyle(wxBG_STYLE_CUSTOM);
}

BEGIN_EVENT_TABLE(wxGLPane, wxGLCanvas)
	EVT_MOTION(wxGLPane::mouseMoved)
	EVT_LEFT_DOWN(wxGLPane::leftMouseDown)
	EVT_LEFT_UP(wxGLPane::leftMouseUp)
	EVT_KEY_DOWN(wxGLPane::keyPressed)
	EVT_KEY_UP(wxGLPane::keyReleased)
	EVT_TIMER(0, wxGLPane::onDisplayTimer)
	//EVT_LEFT_UP(BasicGLPane::mouseReleased)
	//EVT_RIGHT_DOWN(wxGLPane::OnRightClick)
	//EVT_LEFT_DOWN(wxGLPane::OnLeftClick)
	//EVT_LEAVE_WINDOW(BasicGLPane::mouseLeftWindow)
	EVT_SIZE(wxGLPane::reshape)
	//EVT_MOUSEWHEEL(BasicGLPane::mouseWheelMoved)
	EVT_PAINT(wxGLPane::display)
END_EVENT_TABLE()

void wxGLPane::reshape(wxSizeEvent& evt)
{
	Refresh();
	if(m_firstFrame) return; 
	m_chagApp->onReshape(evt.GetSize().x, evt.GetSize().y);
}

void wxGLPane::display(wxPaintEvent &evt) 
{
	if(!IsShown()) return;
	wxGLCanvas::SetCurrent(*m_context);
	wxPaintDC(this); // only to be used in paint events. use wxClientDC 
					 // to paint outside the paint event

	///////////////////////////////////////////////////////////////////////////
	// TODO: Very annoyingly, my application does not seem to want to honor
	//       the Close() call, for a long time. This is part of a quick fix 
	//		 that should be improved with whatever is the right way to do it. 
	///////////////////////////////////////////////////////////////////////////
	if(m_chagApp->m_closeRequested) return; 

	///////////////////////////////////////////////////////////////////////////
	// Reshape event doesn't happen often enough (or too often.. never mind)
	///////////////////////////////////////////////////////////////////////////
	int w = GetSize().x;
	int h = GetSize().y;
	m_chagApp->m_gl_pane_width = w; 
	m_chagApp->m_gl_pane_height = h; 
	glViewport(0, 0, w, h); 

	//////////////////////////////////////////////////////////////////////////////
	// We may not want this here (if we are rendering a view with a set aspect 
	// ratio), so we kinda need an "aspect_ratio = auto" setting or something. 
	// We don't right now though so... 
	//////////////////////////////////////////////////////////////////////////////
	if(m_chagApp->m_scene != NULL) {
		Scene::PerspectiveCamera *camera = dynamic_cast<Scene::PerspectiveCamera *>(m_chagApp->m_scene->getCurrentRenderableItem());
		if(camera != NULL) {
			camera->m_aspect_ratio = float(GetSize().x) / float(GetSize().y);
		}
	}

	///////////////////////////////////////////////////////////////////////////
	// Set up stuff I tend to assume to be GL default but isn't 
	///////////////////////////////////////////////////////////////////////////
	glEnable(GL_DEPTH_TEST); 

	///////////////////////////////////////////////////////////////////////////
	// For now, assume we get our full expected fps for motion. Will meassuare
	// later. 
	///////////////////////////////////////////////////////////////////////////
	if(m_chagApp->m_scene != NULL) {
		float dt = CHAGAPP_DISPLAY_TIMER_MS; 
		float boost = m_shiftDown ? 0.4f : 0.1f; 
		float sceneUnitMultiplier = m_chagApp->m_scene->m_sceneUnitMultiplier; 
		chag::orientation *camera = &m_chagApp->m_scene->getCurrentControllableItem()->m_transform; 
		camera->pos += boost * sceneUnitMultiplier * dt * m_forwardVelocity * camera->R.c3; 
		camera->pos += boost * sceneUnitMultiplier * dt * m_strafeVelocity * camera->R.c1; 
		camera->pos += boost * sceneUnitMultiplier * dt * m_ascentVelocity * camera->R.c2; 
		if(m_forwardVelocity != 0.0f || m_strafeVelocity != 0.0f || m_ascentVelocity != 0.0f){
			m_chagApp->onViewChanged(); 
		}
	}

	if(m_firstFrame) {
		m_chagApp->firstFrame();
		m_chagApp->onReshape(w, h); 
	}
	if(m_chagApp->m_scene != NULL || !m_chagApp->applicationLoadsScene()) m_chagApp->onDisplay(); 
	m_firstFrame = false; 

	SwapBuffers();
	m_frame_finished = true; 
}

void wxGLPane::keyPressed(wxKeyEvent &event)
{
	m_shiftDown = event.ShiftDown();
	switch(event.GetKeyCode()) {
	case 'S': m_forwardVelocity =  1.0f; break;
	case 'W': m_forwardVelocity = -1.0f; break;
	case 'D': m_strafeVelocity	=  1.0f; break;
	case 'A': m_strafeVelocity	= -1.0f; break;
	case 'E': m_ascentVelocity	=  1.0f; break;
	case 'Q': m_ascentVelocity	= -1.0f; break;
	}
}

void wxGLPane::keyReleased(wxKeyEvent &event)
{
	m_shiftDown = event.ShiftDown();
	switch(event.GetKeyCode()) {
	case 'S': m_forwardVelocity = 0.0f; break;
	case 'W': m_forwardVelocity = 0.0f; break;
	case 'D': m_strafeVelocity	= 0.0f; break;
	case 'A': m_strafeVelocity	= 0.0f; break;
	case 'E': m_ascentVelocity	= 0.0f; break;
	case 'Q': m_ascentVelocity	= 0.0f; break;
	}
}

void wxGLPane::mouseMoved(wxMouseEvent &event)
{
	if(m_firstFrame) return; 
	m_chagApp->onMouse(event.GetX(), GetSize().y - event.GetY() - 1, event.LeftIsDown(), event.MiddleIsDown(), event.RightIsDown()); 

	if(m_chagApp->m_scene == NULL) return; 
	float delta_x = m_mouseSensitivity * 0.1f * (m_oldMouseX - event.GetX());
	float delta_y = m_mouseSensitivity * 0.1f * (m_oldMouseY - event.GetY());		

	chag::orientation &transform = m_chagApp->m_scene->getCurrentControllableItem()->m_transform;

	if(event.LeftIsDown() && event.RightIsDown()) { // Zoom
		transform.pos -= delta_y * transform.R.c3; 
		m_chagApp->onViewChanged(); 
	}
	else if(event.LeftIsDown()) {
		transform.pitch(delta_y * 0.05f);
		transform.yaw(delta_x * 0.05f);
		m_chagApp->onViewChanged(); 
	}
	else if(event.RightIsDown()) {
		transform.roll(delta_x * 0.05f);
		m_chagApp->onViewChanged(); 
	}
	else if(event.MiddleIsDown()){
		transform.pos -= (delta_y * m_chagApp->m_scene->m_sceneUnitMultiplier) * transform.R.c2; 
		transform.pos += (delta_x * m_chagApp->m_scene->m_sceneUnitMultiplier) * transform.R.c1; 
		m_chagApp->onViewChanged(); 
	}
	m_oldMouseX = event.GetX();
	m_oldMouseY = event.GetY(); 
}

void wxGLPane::leftMouseDown(wxMouseEvent &event)
{
	SetFocus(); 
	m_oldMouseX = event.GetX(); 
	m_oldMouseY = event.GetY(); 
	m_chagApp->onMouse(m_oldMouseX, GetSize().y - m_oldMouseY - 1, event.LeftIsDown(), event.MiddleIsDown(), event.RightIsDown()); 
};

void wxGLPane::leftMouseUp(wxMouseEvent &event)
{
	SetFocus(); 
	m_oldMouseX = event.GetX(); 
	m_oldMouseY = event.GetY(); 
	m_chagApp->onMouse(m_oldMouseX, GetSize().y - m_oldMouseY - 1, event.LeftIsDown(), event.MiddleIsDown(), event.RightIsDown()); 
};
