#include "CompressedShadow.h"
#include <utils/Log.h>
#include <fstream>
#include <linmath/Aabb.h>
#include <linmath/streamoperators.h>
#include <utils/DebugRenderer.h>
#include <utils/Log.h>
#include <assert.h>

using namespace std; 

void CompressedShadow::load(const std::string &filename)
{
	m_file_name = filename;
	ifstream is(filename.c_str(), ios::binary); 
	if(!is.is_open()) LOG_ERROR("Failed to open inputfile " << filename); 
	is.read((char *)&m_levels, sizeof(unsigned int)); 
	is.read((char *)&m_data_size, sizeof(unsigned int)); 
	m_data = new unsigned int[m_data_size]; 
	for(int i=0; i<m_data_size; i++) {
		is.read((char *)&m_data[i], sizeof(unsigned int)); 
	}
	is.close(); 
	LOG_INFO("Read " << filename << ". Size: " << m_data_size * 4 << ". Levels: " << m_levels );
	buildTopGrid(); 
	uploadData(); 
}

void CompressedShadow::buildTopGrid()
{
	unsigned int res = (1 << (TOPGRID_LEVELS - 1)); 
	LOG_INFO("Building grid of first " << TOPGRID_LEVELS << " levels (" << res*res*res*sizeof(unsigned int) << " bytes).");
	m_topGrid = new unsigned int[res*res*res];
	for(int z=0; z<res; z++) {
		for(int y=0; y<res; y++) {
			for(int x=0; x<res; x++) {
				chag::int3 path = chag::make_vector(x,y,z);
				unsigned int level = 0; 
				unsigned int offset = 0;
				while(level <= (TOPGRID_LEVELS - 2)) {
					unsigned char childIndex = (((path.x & (1<<(TOPGRID_LEVELS-level-2))) == 0) ? 0 : 1) << 0 | 
											   (((path.y & (1<<(TOPGRID_LEVELS-level-2))) == 0) ? 0 : 1) << 1 | 
											   (((path.z & (1<<(TOPGRID_LEVELS-level-2))) == 0) ? 0 : 1) << 2; 
					{
						unsigned int childMask = m_data[offset];
						unsigned char childIsectResult =  0x3 & (childMask >> (childIndex*2)); 
						if(childIsectResult == 0x0) { // NODE_IN_SHADOW
							offset = 0xFFFFFFFF;
							break;
						}
						else if (childIsectResult == 0x1) { // NODE_VISIBLE
							offset = 0xFFFFFFFE;
							break;
						}
						else if (childIsectResult == 0x2){ // NODE_INTERSECTS
							unsigned int maskedChildMask = (childMask & 0xAAAA) & (0xFFFF >> ((8-childIndex) * 2)); 
							unsigned int childOffset = __popcnt(maskedChildMask);
							offset = m_data[offset + 1 + childOffset]; 
						}
						// Note: UNDEFINED is not a case for a finished DAG
					}
					level += 1;
				}
				m_topGrid[z*res*res+y*res+x] = offset; 
			}
		}
	}
}

