#include <wx/wxprec.h>
#include <iostream>
#include "../CHAGApp/CHAGApp.h"
#include <utils/GLSLProgramManager.h>
#include "CPVSBuilder.h"
#include "CompressedShadow.h"
#include <GL/freeglut.h>
#include <sstream>

using namespace std; 
using namespace chag; 

class NewApplication : public CHAGApp
{
	GLSLProgramManager *m_programManager; 
	GLSLProgramObject  *m_gbuffer_program; 
	GLSLProgramObject  *m_deferred_shading_program;
	GLSLProgramObject  *m_simple_program;
public:
	virtual void onFirstFrame() {
		// WARNING: m_scene can still be NULL here
		glewInit(); 
		int argc = 0;
		char **argv = NULL;
		glutInit(&argc, argv);
		m_programManager = new GLSLProgramManager("Application.glsl"); 
		m_gbuffer_program = m_programManager->getProgramObject("gbuffer_vertex", "gbuffer_fragment"); 
		m_deferred_shading_program = m_programManager->getProgramObject("deferred_shading_vertex", "deferred_shading_fragment"); 
		m_simple_program = m_programManager->getProgramObject("simpleVertex", "simpleFragment");
	}

	virtual void onReshape(int w, int h) {
		CompressedShadow::unregisterTextures(); 
		m_gbuffer.resize(w, h); 
		CompressedShadow::registerTextures(m_gbuffer.m_shadow_result_texture, m_gbuffer.m_position_texture, m_gbuffer.m_normal_texture); 
		CompressedShadow::resize(w, h); 
	}

	virtual void onDisplay() {
		glPushAttrib(GL_ALL_ATTRIB_BITS); 
		glViewport(0, 0, m_gl_pane_width, m_gl_pane_height); 
		view &currentCamera = m_scene->getCurrentRenderableItem()->getView(); 

		m_scene->updateShadowGenerators(); 
		///////////////////////////////////////////////////////////////////////
		// Render G-Buffer
		///////////////////////////////////////////////////////////////////////
		glBindFramebuffer(GL_FRAMEBUFFER, m_gbuffer.m_framebuffer_id); 
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f); 
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
		currentCamera.push();
		m_gbuffer_program->enable(); 
		m_scene->submitModels(m_gbuffer_program); 
		m_gbuffer_program->disable(); 
		currentCamera.pop(); 

		///////////////////////////////////////////////////////////////////////
		// Shade, for each light
		///////////////////////////////////////////////////////////////////////
		glBindFramebuffer(GL_FRAMEBUFFER, 0); 
		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glBlendFunc(GL_ONE, GL_ONE); 
		glEnable(GL_BLEND); 
		glDisable(GL_DEPTH_TEST); 
		m_deferred_shading_program->enable(); 
		for(auto light : m_scene->m_lights) {
 			Scene::ShadowGenerator *shadow_generator = NULL; 
			for(auto sg : m_scene->m_shadowGenerators) {
				if(sg->m_light == light) shadow_generator = sg; 
			}

			Scene::ShadowMapShadowGenerator *shadow_map_generator = dynamic_cast<Scene::ShadowMapShadowGenerator *>(shadow_generator);
			CompressedShadow *compressed_shadow = dynamic_cast<CompressedShadow *>(shadow_generator); 
			m_deferred_shading_program->setUniform("light_has_shadowmap", false);
			m_deferred_shading_program->setUniform("light_has_compressed_shadow", false);
			if(compressed_shadow) {
				///////////////////////////////////////////////////////////////////////
				// Render compressed shadow result
				///////////////////////////////////////////////////////////////////////
				compressed_shadow->renderShadows(compressed_shadow->getView().get_MVP()); 
				m_deferred_shading_program->setUniform("light_has_compressed_shadow", true); 
				m_deferred_shading_program->bindTexture("light_compressed_shadow_result_texture", m_gbuffer.m_shadow_result_texture, GL_TEXTURE_2D, 4); 
			}
			else if(shadow_map_generator) {
				m_deferred_shading_program->setUniform("light_has_shadowmap", true);
				m_deferred_shading_program->bindTexture("light_shadowmap_sampler", shadow_map_generator->m_depth_texture, GL_TEXTURE_2D, 3);
				m_deferred_shading_program->setUniform("light_shadowmap_mvp", shadow_map_generator->getView().get_MVP()); 				
			}

			Scene::DiscDiffuseLight *ddl = dynamic_cast<Scene::DiscDiffuseLight *>(light);
			m_deferred_shading_program->setUniform("light_position", ddl->m_transform.pos); 
			m_deferred_shading_program->setUniform("light_direction", -ddl->m_transform.R.c3); 
			m_deferred_shading_program->setUniform("light_radiance", ddl->m_radiance * ddl->m_color); 
			m_deferred_shading_program->setUniform("light_radius", ddl->m_radius); 
			m_deferred_shading_program->bindTexture("position_texture", m_gbuffer.m_position_texture, GL_TEXTURE_2D, 0); 
			m_deferred_shading_program->bindTexture("normal_texture", m_gbuffer.m_normal_texture, GL_TEXTURE_2D, 1); 
			m_deferred_shading_program->bindTexture("diffuse_reflectance_texture", m_gbuffer.m_diffuse_reflectance_texture, GL_TEXTURE_2D, 2); 
			view::drawFullScreenQuad(); 

			///////////////////////////////////////////////////////////////////////
			// One final check for GL errors
			///////////////////////////////////////////////////////////////////////
			GLenum gl_error = glGetError(); 
			if(gl_error != GL_NO_ERROR) LOG_ERROR(gluErrorString(gl_error));
		}
		m_deferred_shading_program->disable(); 


		///////////////////////////////////////////////////////////////////////
		// Copy the depth of the G-Buffer, to allow for overlay graphics
		///////////////////////////////////////////////////////////////////////
		glBindFramebuffer(GL_READ_FRAMEBUFFER, m_gbuffer.m_framebuffer_id);
		glBlitFramebuffer(0, 0, m_gbuffer.m_width, m_gbuffer.m_height,
			0, 0, m_gl_pane_width, m_gl_pane_height,
			GL_DEPTH_BUFFER_BIT, GL_NEAREST);
		glBindFramebuffer(GL_FRAMEBUFFER, 0);

		///////////////////////////////////////////////////////////////////////
		// Render the light sources
		///////////////////////////////////////////////////////////////////////
		currentCamera.push();
		m_simple_program->enable();
		m_scene->submitLightGeometry(m_simple_program);
		m_simple_program->disable();
		currentCamera.pop();

		///////////////////////////////////////////////////////////////////////
		// Display the meassured time for shadow lookups
		///////////////////////////////////////////////////////////////////////
		int i = 0; 
		glDisable(GL_DEPTH_TEST);
		for (auto sg : m_scene->m_shadowGenerators) {
			glMatrixMode(GL_PROJECTION); 
			glLoadIdentity(); 
			glOrtho(0, m_gl_pane_width, 0, m_gl_pane_height, -1.0f, 1.0f);
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity(); 
			CompressedShadow *compressed_shadow = dynamic_cast<CompressedShadow *>(sg);
			if (compressed_shadow) {
				stringstream ss; 
				ss << compressed_shadow->m_name << " : " << compressed_shadow->m_meassured_time; 
				glWindowPos2i(10, 10 + i++ * 18); 
				glutBitmapString(GLUT_BITMAP_9_BY_15, (const unsigned char *)ss.str().c_str()); 
			}
		}
		glPopAttrib(); 
	}; 

	///////////////////////////////////////////////////////////////////////////
	// A class representing a GBuffer. We use full-blown float32 rgba textures
	// for everything just to keep things simple. 
	///////////////////////////////////////////////////////////////////////////
	class GBuffer {
	public:
		GLuint	m_framebuffer_id;
		GLuint	m_position_texture;
		GLuint	m_normal_texture;
		GLuint	m_diffuse_reflectance_texture;
		GLuint	m_depth_texture;

		GLuint	m_shadow_result_texture;

		int		m_width, m_height;
		GBuffer() { m_framebuffer_id = 0; };
		void resize(int w, int h) {
			m_width = w;
			m_height = h;
			if (m_framebuffer_id == 0){
				glGenFramebuffers(1, &m_framebuffer_id);
				glGenTextures(1, &m_position_texture);
				glGenTextures(1, &m_normal_texture);
				glGenTextures(1, &m_diffuse_reflectance_texture);
				glGenTextures(1, &m_depth_texture);
				glGenTextures(1, &m_shadow_result_texture);
			}
			glBindTexture(GL_TEXTURE_2D, m_position_texture);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA32F, w, h, 0, GL_RGBA, GL_FLOAT, NULL);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glBindTexture(GL_TEXTURE_2D, m_normal_texture);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA32F, w, h, 0, GL_RGBA, GL_FLOAT, NULL);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glBindTexture(GL_TEXTURE_2D, m_diffuse_reflectance_texture);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA32F, w, h, 0, GL_RGBA, GL_FLOAT, NULL);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glBindTexture(GL_TEXTURE_2D, m_depth_texture);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT32, w, h, 0, GL_DEPTH_COMPONENT, GL_FLOAT, NULL);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glBindFramebuffer(GL_FRAMEBUFFER, m_framebuffer_id);
			glFramebufferTexture2D(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, m_position_texture, 0);
			glFramebufferTexture2D(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT1, GL_TEXTURE_2D, m_normal_texture, 0);
			glFramebufferTexture2D(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT2, GL_TEXTURE_2D, m_diffuse_reflectance_texture, 0);
			glFramebufferTexture2D(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT, GL_TEXTURE_2D, m_depth_texture, 0);
			GLenum drawbuffers[] = { GL_COLOR_ATTACHMENT0, GL_COLOR_ATTACHMENT1, GL_COLOR_ATTACHMENT2 };
			glDrawBuffers(3, drawbuffers);
			glBindFramebuffer(GL_FRAMEBUFFER, 0);

			// Add a "shadow result" buffer which should always be of gbuffer size but is rendered
			// to in a different (CUDA) pass. 			
			glBindTexture(GL_TEXTURE_2D, m_shadow_result_texture);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_R32F, w, h, 0, GL_RED, GL_FLOAT, NULL);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		}
	} m_gbuffer;


	///////////////////////////////////////////////////////////////////////////
	// Blow is stuff only related to our wxWidgets UI. Can be safely ignored
	// by most people. 
	///////////////////////////////////////////////////////////////////////////
	virtual void onApplicationCreateMenuBar(wxMenuBar *menuBar);
	virtual void onUnhandledMenuCommand(wxCommandEvent &event);
	virtual bool handleUnknownType(std::string &type, std::istream &is, Scene *scene)
	{
		if (type == "compressedshadowgenerator") {
			CompressedShadow *cs = new CompressedShadow();
			is >> cs->m_name >> cs->m_file_name;
			cs->load(cs->m_file_name);
			string lightName;
			is >> lightName >> cs->m_fov >> cs->m_aspect_ratio >> cs->m_near >> cs->m_far >> cs->m_bias >> cs->m_filter_size;
			cs->m_light = NULL;
			for (auto l : m_scene->m_lights) if (l->m_name == lightName) cs->m_light = l;
			if (cs->m_light == NULL) {
				LOG_ERROR("Scene does not contain light: " << lightName);
				return false;
			}
			m_scene->m_shadowGenerators.push_back(cs);
			onReshape(m_gl_pane_width, m_gl_pane_height);
			return true;
		}
		return true;
	};
	virtual void OnSceneTreeCtrlSelectionChanged(wxTreeEvent &event) {
		wxTreeItemId selected = m_sceneTreeCtrl->GetSelection();
		Scene::Item * item = m_sceneItems.count(selected) > 0 ? m_sceneItems[selected] : NULL;
		CompressedShadow * shadowGenerator = dynamic_cast<CompressedShadow *>(item);
		if (shadowGenerator) {
			m_sceneItemPropertyGrid->Clear();
			m_sceneItemPropertyGrid->Append(new wxPropertyCategory("Compressed Shadow"));
			m_sceneItemPropertyGrid->Append(new wxFloatProperty("Bias", wxPG_LABEL, shadowGenerator->m_bias));
			m_sceneItemPropertyGrid->Append(new wxIntProperty("FilterSize", wxPG_LABEL, shadowGenerator->m_filter_size));
		}
		else CHAGApp::OnSceneTreeCtrlSelectionChanged(event);
	}
	virtual void initializeSceneTreeCtrl() {
		CHAGApp::initializeSceneTreeCtrl();
		LOG_INFO("Application initialize scene tree ctrl");
	}
	virtual void OnSceneTreeCtrlMenu(wxTreeEvent &event) {
		CHAGApp::OnSceneTreeCtrlMenu(event);
		LOG_INFO("Application on scene tree ctrl menu");
	}
	virtual void OnScenePropertyGridChanged(wxPropertyGridEvent &event) {
		wxTreeItemId selected = m_sceneTreeCtrl->GetSelection();
		Scene::Item * item = m_sceneItems[selected];
		CompressedShadow * shadowGenerator = dynamic_cast<CompressedShadow *>(item);
		wxAny value = event.GetValue();
		if (shadowGenerator) {
			if (event.GetPropertyName() == "Bias") shadowGenerator->m_bias = value.As<float>();
			if (event.GetPropertyName() == "FilterSize") shadowGenerator->m_filter_size = value.As<int>();
		}
		else {
			CHAGApp::OnScenePropertyGridChanged(event);
		}
	}
	virtual void onReloadShaders() {
		m_programManager->reload();
	};
	virtual void onSceneChanged()	{
	};
	virtual void onViewChanged() {
	};
};

const int ID_BuildCPVS = 100;
class BuildCPVSDialog : public wxDialog
{
public:
	wxComboBox *m_resolutionComboBox;
	wxCheckBox *m_closedObjectsCheckBox;
	BuildCPVSDialog(wxWindow * parent) : wxDialog(parent, -1, "Build CPVS", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE) {
		wxBoxSizer *vsizer = new wxBoxSizer(wxVERTICAL);
		vsizer->Add(new wxStaticText(this, -1, "Resolution:"));
		wxString choices[7] = { "4096", "8192", "16536", "32768", "65536", "131072", "262144" };
		m_resolutionComboBox = new wxComboBox(this, -1, "4096", wxDefaultPosition, wxDefaultSize, 7, choices, wxCB_READONLY);
		vsizer->Add(m_resolutionComboBox);
		m_closedObjectsCheckBox = new wxCheckBox(this, -1, "Closed Objects");
		vsizer->Add(m_closedObjectsCheckBox);
		wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);
		hsizer->Add(new wxButton(this, wxID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize));
		hsizer->Add(new wxButton(this, wxID_OK, _("OK"), wxDefaultPosition, wxDefaultSize));
		vsizer->Add(hsizer);
		SetSizerAndFit(vsizer);
	}
	bool useClosedObjects() { return m_closedObjectsCheckBox->GetValue(); };
	int  levels() {
		if (m_resolutionComboBox->GetValue() == "4096") return 13;
		if (m_resolutionComboBox->GetValue() == "8192") return 14;
		if (m_resolutionComboBox->GetValue() == "16536") return 15;
		if (m_resolutionComboBox->GetValue() == "32768") return 16;
		if (m_resolutionComboBox->GetValue() == "65536") return 17;
		if (m_resolutionComboBox->GetValue() == "131072") return 18;
		if (m_resolutionComboBox->GetValue() == "262144") return 19;
		return -1;
	}
private:
	DECLARE_EVENT_TABLE()
};
BEGIN_EVENT_TABLE(BuildCPVSDialog, wxDialog)
END_EVENT_TABLE()

void NewApplication::onApplicationCreateMenuBar(wxMenuBar *menuBar)
{
	wxMenu *myMenu = new wxMenu;
	myMenu->Append(ID_BuildCPVS, "&Build CPVS...\tCTRL+B", "Build a CPVS from the selected shadow map");
	menuBar->Append(myMenu, "Application");
}

void NewApplication::onUnhandledMenuCommand(wxCommandEvent &event) 
{
	if (event.GetId() == ID_BuildCPVS) {
		Scene::ShadowMapShadowGenerator *shadowGenerator = dynamic_cast<Scene::ShadowMapShadowGenerator *>(m_sceneItems[m_sceneTreeCtrl->GetSelection()]);
		if (shadowGenerator) {
			LOG_INFO("Build CPVS");
			BuildCPVSDialog *dlg = new BuildCPVSDialog(m_frame);
			if (dlg->ShowModal() == wxID_OK) {
				CPVSBuilder builder;
				wxFileDialog saveFileDialog(m_frame, _("Save CSHADOW file"), "", "",
					"CSHADOW files (*.cshadow)|*.cshadow", wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
				if (saveFileDialog.ShowModal() == wxID_CANCEL) return;
				builder.build(m_scene, shadowGenerator, dlg->levels(), dlg->useClosedObjects());
				std::string filename = saveFileDialog.GetPath();
				builder.saveCompactData(filename);
				// Replace shadowmapshadowgenerator with new thing
				CompressedShadow *cs = new CompressedShadow();
				cs->load(filename);

				for (int i = 0; i<m_scene->m_shadowGenerators.size(); i++) {
					Scene::ShadowGenerator *sg = m_scene->m_shadowGenerators[i];
					if (sg == shadowGenerator) {
						LOG_INFO("Replacing ShadowMapShadowGenerator");
						m_scene->m_shadowGenerators.erase(m_scene->m_shadowGenerators.begin() + i);
						//cs->m_original_shadow_map = shadowGenerator; 
						cs->m_light = shadowGenerator->m_light;
						cs->m_name = "CompressedShadow::" + cs->m_light->m_name;
						cs->m_near = shadowGenerator->m_near;
						cs->m_far = shadowGenerator->m_far;
						cs->m_aspect_ratio = shadowGenerator->m_aspect_ratio;
						cs->m_fov = shadowGenerator->m_fov;
						m_scene->m_shadowGenerators.push_back(cs);
					}
				}
				onReshape(m_gl_pane_width, m_gl_pane_height);
				initializeSceneTreeCtrl();
			}
			else LOG_INFO("Cancel");
		}
		else { LOG_INFO("Select a shadow map shadow generator."); }
	}
	else { LOG_INFO("No Handler."); }
};

wxIMPLEMENT_APP(NewApplication);
