#include "Animation.h"

using namespace std; 
using namespace chag; 

Animation::Animation(void)
{
}

Animation::~Animation(void)
{
}

void Animation::load(std::string filename)
{
	///////////////////////////////////////////////////////////////////////////
	// Establish filename and path and open file
	///////////////////////////////////////////////////////////////////////////
	ifstream file;
	file.open(filename.c_str());
	if (!file) {
		cout << "Failed to open animation " << filename;
		assert(0); 
	}
	///////////////////////////////////////////////////////////////////////////
	// Parse file
	///////////////////////////////////////////////////////////////////////////
	int start, end; 
	file >> start >> end; 
	m_nofFrames = end - start + 1; 
	string name, parentname; 
	while(!file.eof()){
		// Removing "" from strings
		file >> name; 
		if(file.eof() || name == "") break;
		name = name.substr(1, name.length() - 2);  

		if(name == "freespot")
		{
			lightTransform tfm; 
			file >> tfm.m_fov;
			if(tfm.m_fov == 360.0f) tfm.m_fov = 0.0f; 
			file >> tfm.m_aspectratio; 
			file >> tfm.m_color.x >> tfm.m_color.y >> tfm.m_color.z;
			tfm.m_color *= (1.0f/256.0f);
			file >> tfm.m_intensity; 
			tfm.m_intensity *= 100.0f; 
			file >> name; 
			name = name.substr(1, name.length() - 2);
			file >> parentname; 
			tfm.m_parentname = parentname.substr(1, parentname.length() - 2); 
			for(int i=0; i<m_nofFrames; i++)
			{
				file >> tfm.m_pos.x >> tfm.m_pos.y >> tfm.m_pos.z; 
				file >> tfm.m_rot.v.x >> tfm.m_rot.v.y >> tfm.m_rot.v.z >> tfm.m_rot.w; 
				m_lights[name].push_back(tfm);
			}
		}
		else if(name == "freecamera")
		{
			cameraTransform tfm; 
			file >> tfm.m_fov;
			file >> name; 
			name = name.substr(1, name.length() - 2);
			file >> parentname; 
			tfm.m_parentname = parentname.substr(1, parentname.length() - 2); 
			for(int i=0; i<m_nofFrames; i++)
			{
				file >> tfm.m_pos.x >> tfm.m_pos.y >> tfm.m_pos.z; 
				file >> tfm.m_rot.v.x >> tfm.m_rot.v.y >> tfm.m_rot.v.z >> tfm.m_rot.w; 
				m_cameras[name].push_back(tfm);
			}
		}
		else 
		{
			transform tfm; 
			file >> parentname; 
			tfm.m_parentname = parentname.substr(1, parentname.length() - 2); 
			for(int i=0; i<m_nofFrames; i++)
			{
				file >> tfm.m_pos.x >> tfm.m_pos.y >> tfm.m_pos.z; 
				file >> tfm.m_rot.v.x >> tfm.m_rot.v.y >> tfm.m_rot.v.z >> tfm.m_rot.w; 
				m_nodes[name].push_back(tfm);
			}
		}
	}
	file.close(); 
}

vector<string> Animation::getCameraNames()
{
	vector<string> names; 
	for(map<string, vector<cameraTransform>>::iterator iter = m_cameras.begin(); iter != m_cameras.end(); iter++)
	{
		names.push_back(iter->first);
	}
	return names; 
}

vector<string> Animation::getLightNames()
{
	vector<string> names; 
	for(map<string, vector<lightTransform>>::iterator iter = m_lights.begin(); iter != m_lights.end(); iter++)
	{
		names.push_back(iter->first);
	}
	return names; 
}

vector<string> Animation::getNodeNames()
{
	vector<string> names; 
	for(map<string, vector<transform>>::iterator iter = m_nodes.begin(); iter != m_nodes.end(); iter++)
	{
		names.push_back(iter->first);
	}
	return names; 
}

Light Animation::getLight(string name, float time)
{
	int currframe = int(floor(time)) % m_nofFrames; 
	int nextframe = int(ceil(time)) % m_nofFrames; 
	lightTransform curr =  m_lights[name][currframe];
	lightTransform next =  m_lights[name][nextframe];

	float frac = time - floor(time); 
	Light l;
	l.myView.pos = (1.0f - frac) *curr.m_pos + frac * next.m_pos;
	l.myView.R = toMatrix3x3(slerp(curr.m_rot, next.m_rot, frac));
	l.myView.m_aspect_ratio = (1.0f - frac) * curr.m_aspectratio + frac * next.m_aspectratio;
	l.myView.m_fov = (1.0f - frac) * curr.m_fov + frac * next.m_fov;
	l.color = (1.0f - frac) * curr.m_color + frac * next.m_color; 
	l.intensity = (1.0f - frac) * curr.m_intensity + frac * next.m_intensity; 
	l.spotlight_total_width = l.myView.m_fov * (3.14152f/180.0f); 
	l.spotlight_falloff_start = 0.9f * l.myView.m_fov * (3.14152f/180.0f); 
	return l; 
}

view Animation::getCamera(string name, float time)
{
	int currframe = int(floor(time)) % m_nofFrames; 
	int nextframe = int(ceil(time)) % m_nofFrames; 
	if(currframe == nextframe) nextframe = (currframe + 1) % m_nofFrames; 
	cameraTransform curr =  m_cameras[name][currframe];
	cameraTransform next =  m_cameras[name][nextframe];
	float frac = time - floor(time); 
	view c;
	c.pos = (1.0f - frac) *curr.m_pos + frac * next.m_pos;
	c.R = toMatrix3x3(slerp(curr.m_rot, next.m_rot, frac));
	c.m_aspect_ratio = 1.0f; 
	c.m_fov = (1.0f - frac) * curr.m_fov + frac * next.m_fov;
	return c; 
}

view Animation::getNode(string name, float time)
{
	int currframe = int(floor(time)) % m_nofFrames; 
	int nextframe = int(ceil(time)) % m_nofFrames; 
	if(currframe == nextframe) nextframe = (currframe + 1) % m_nofFrames; 
	transform curr =  m_nodes[name][currframe];
	transform next =  m_nodes[name][nextframe];
	float frac = time - floor(time); 
	view c;
	c.pos = (1.0f - frac) *curr.m_pos + frac * next.m_pos;
	c.R = toMatrix3x3(slerp(curr.m_rot, next.m_rot, frac));
	c.m_aspect_ratio = 1.0f; 
	c.m_fov = 45.0f; 
	return c; 
}