#include "CameraController.h"
#include <GL/freeglut.h>
using namespace chag; 

CameraController::CameraController(void)
{
	m_oldMouseX = 0; 
	m_oldMouseY = 0;
	m_mouseSensitivity = 1.0f; 
	m_mouseRightPressed = false; 
	m_mouseLeftPressed = false; 
	m_mouseMiddlePressed = false; 
	m_ctrlPressed = false; 
	m_controllerType = FREE; 
	m_forwardVelocity = 0.0f; 
	m_strafeVelocity = 0.0f; 
	m_ascentVelocity = 0.0f; 
	m_sceneUnitMultiplier = NULL; 
}

CameraController::~CameraController(void)
{
}

void CameraController::setCamera(view *camera)
{
	m_camera = camera; 
}

void CameraController::setTarget(Orientation *target)
{
	m_target = target; 
}

bool CameraController::keyboard(unsigned char c, int x, int y, bool down)
{
	switch(c) {
	case '1':
		m_controllerType = FREE; 
		break;
	case '2':
		m_controllerType = ORBITING; 
		m_camera->lookAt(m_camera->pos, m_target->pos, m_target->R.c2); 
		break;
	case 'w': 
	case 'W':
		m_forwardVelocity = down ? -1.0f : 0.0f; 
		break;
	case 's':
	case 'S':
		m_forwardVelocity = down ? 1.0f : 0.0f;
		break;
	case 'a':
	case 'A':
		m_strafeVelocity = down ? -1.0f : 0.0f; 
		break;
	case 'd':
	case 'D':
		m_strafeVelocity = down ? 1.0f : 0.0f;
		break;
	case 'q':
	case 'Q':
		m_ascentVelocity = down ? -1.0f : 0.0f; 
		break;
	case 'e':
	case 'E':
		m_ascentVelocity = down ? 1.0f : 0.0f;
		break;
	}
	return false; 
}

bool CameraController::isShiftPressed()
{
  return (( GetKeyState( VK_LSHIFT   ) < 0 ) || ( GetKeyState( VK_RSHIFT   ) < 0 ));
}

bool CameraController::mouse(int button, int state, int x, int y)
{
	if(button == GLUT_RIGHT_BUTTON) {
		m_mouseRightPressed = state == GLUT_DOWN;
	}
	if(button == GLUT_LEFT_BUTTON) {
		m_mouseLeftPressed = state == GLUT_DOWN;
	}
	if(button == GLUT_MIDDLE_BUTTON) {
		m_mouseMiddlePressed = state == GLUT_DOWN;
	}
	// User must hold down Ctrl while mouse button is pressed.
	// (To fix, would need to set glutKeyboard{Up,Down}Func, ehhh.)
	m_ctrlPressed = (glutGetModifiers() & GLUT_ACTIVE_CTRL) != 0;
	m_oldMouseX = x;
	m_oldMouseY = y; 
	return false; 
}

bool CameraController::motion(int x, int y)
{
	float delta_x = m_mouseSensitivity * 0.1f * (m_oldMouseX - x);
	float delta_y = m_mouseSensitivity * 0.1f * (m_oldMouseY - y);
	float sceneUnitMultiplier = m_sceneUnitMultiplier == NULL ? 1.0f : *m_sceneUnitMultiplier; 

	switch(m_controllerType){
		case FREE: 
			if(m_mouseRightPressed && m_mouseLeftPressed) // Zoom
			{
				m_camera->pos -= delta_y * m_camera->R.c3;		
			}
			else if(m_mouseLeftPressed) {
				m_camera->pitch(delta_y * 0.05f);
				m_camera->yaw(delta_x * 0.05f);
			}
			else if(m_mouseRightPressed) {
				m_camera->roll(delta_x * 0.05f);
			}
			else if(m_mouseMiddlePressed){
				m_camera->pos -= (delta_y * sceneUnitMultiplier) * m_camera->R.c2; 
				m_camera->pos += (delta_x * sceneUnitMultiplier) * m_camera->R.c1; 
			}
			break;
		case ORBITING: 
			{
				if(m_camera == m_target) break;
				float3 &m_cameraTarget = m_target->pos; 
				float3 &m_cameraTargetAxis = m_target->R.c2; 
				if(m_mouseLeftPressed && m_mouseRightPressed) { // Zoom on target
					m_camera->pos -= delta_y * 0.1f * sceneUnitMultiplier * (m_cameraTarget - m_camera->pos);
				}
				else if(m_mouseLeftPressed) { // Orbit target
					float dist = length(m_camera->pos - m_cameraTarget);
					m_camera->pos += delta_x * m_camera->R.c1; 
					m_camera->pos = m_cameraTarget + dist * normalize(m_camera->pos - m_cameraTarget);
					m_camera->lookAt(m_camera->pos, m_cameraTarget, m_cameraTargetAxis);
					m_camera->pos -= delta_y * m_cameraTargetAxis; 
					m_camera->pos = m_cameraTarget + dist * normalize(m_camera->pos - m_cameraTarget);
					m_camera->lookAt(m_camera->pos, m_cameraTarget, m_cameraTargetAxis);
				}
				else if(m_mouseMiddlePressed) { // Move target
					m_cameraTarget -= sceneUnitMultiplier * delta_y * m_cameraTargetAxis; 
					m_cameraTarget -= sceneUnitMultiplier * delta_x * normalize(cross(m_cameraTargetAxis, m_camera->pos - m_cameraTarget));
					m_camera->pos -= sceneUnitMultiplier * delta_y * m_cameraTargetAxis; 
					m_camera->pos -= sceneUnitMultiplier * delta_x * normalize(cross(m_cameraTargetAxis, m_camera->pos - m_cameraTarget));
					m_camera->lookAt(m_camera->pos, m_cameraTarget, m_cameraTargetAxis);
				}
				else if(m_mouseRightPressed) { // Move target back and forth
					m_cameraTarget += delta_y * normalize(m_camera->pos - m_cameraTarget);
					m_camera->lookAt(m_camera->pos, m_cameraTarget, m_cameraTargetAxis);
				}
				break;
			}
	}
	m_oldMouseX = x;
	m_oldMouseY = y; 
	return false; 
}

bool CameraController::special(int key, int x, int y)
{
	return false; 
}

bool CameraController::passiveMotion(int x, int y)
{
	m_oldMouseX = x;
	m_oldMouseY = y; 
	return false; 
}

void CameraController::update(float dt)
{
	float boost = isShiftPressed() ? 10.0f : 1.0f; 
	float sceneUnitMultiplier = m_sceneUnitMultiplier == NULL ? 1.0f : *m_sceneUnitMultiplier; 
	m_camera->pos += boost * sceneUnitMultiplier * dt * m_forwardVelocity * m_camera->R.c3; 
	m_camera->pos += boost * sceneUnitMultiplier * dt * m_strafeVelocity * m_camera->R.c1; 
	m_camera->pos += boost * sceneUnitMultiplier * dt * m_ascentVelocity * m_camera->R.c2; 
}

void CameraController::visualize()
{
	/*
	switch(m_controllerType){
		case ORBITING:
			if(m_mouseRightPressed || m_mouseMiddlePressed) {
				glTranslatef(m_cameraTarget.x, m_cameraTarget.y, m_cameraTarget.z);
				glutSolidCube(1.0f);
				glTranslatef(-m_cameraTarget.x, -m_cameraTarget.y, -m_cameraTarget.z);
			}
			break;
	}*/
}