#ifndef _DebugRenderer_h_
#define _DebugRenderer_h_

#include "IntTypes.h"
#include <map>
#include <vector>
#include <string>

#include <linmath/float3.h>
#include <linmath/float4.h>
#include <linmath/float4x4.h>
#include <linmath/Aabb.h>

namespace chag
{

/**
 */
class DebugRenderer
{
public:
	static const chag::float4 red;
	static const chag::float4 blue;
	static const chag::float4 green;
	static const chag::float4 yellow;
	static const chag::float4 purple;
	static const chag::float4 cyan;
	static const chag::float4 orange;
	static const chag::float4 brown;
	static const chag::float4 pink;
	static const chag::float4 olive;


  static DebugRenderer& instance();
  
	void addCone(const float3 &pos, const float3 &dir, float height, float angle, const float4 &color, const std::string &layer = std::string());
	void addAabb(const chag::Aabb &aabb, const float4 &color, const std::string &layer = std::string());
	void addSphere(const float3 &pos, float rad, const float4 &color, const std::string &layer = std::string());
	void addLine(const float3 &pos0, const float3 &pos1, const float4 &color, const std::string &layer = std::string());
	void addTriangle(const float3 &pos0, const float3 &pos1, const float3 &pos2, const float4 &color, const std::string &layer = std::string());

	void clear(const std::string &layer = std::string(), const float4x4 &layerTransform = make_identity<float4x4>());

	void flush(const std::string &layer);

	static chag::float4 randomColor(uint32_t seed, float alpha = 1.0f);
protected:
	virtual void flushLayer(const std::string &layer);

	struct Item
	{
		enum Kind
		{
			K_Cone,
			K_Aabb,
			K_Sphere,
			K_Line,
			K_Triangle,
			K_Max,
		};
		chag::float4 color;
		struct Cone
		{
			chag::float3 position;
			float height;
			chag::float3 direction;
			float angle;
		};
		struct Sphere
		{
			chag::float3 position;
			float radius;
		};
		struct Line
		{
			chag::float3 pos0; 
			chag::float3 pos1; 
		};
		struct Triangle
		{
			chag::float3 pos0; 
			chag::float3 pos1; 
			chag::float3 pos2; 
		};
		union
		{
			Cone cone;
			Aabb aabb;
			Sphere sphere;
			Line line; 
			Triangle triangle; 
		};
		Kind kind;
	};

	struct Layer
	{
		float4x4 transform;
		std::vector<Item> items;
	};

	// map of layers, and their items.
	std::map<std::string, Layer> m_items;

protected: 
  DebugRenderer();
};


}; // namespace chag


#endif // _DebugRenderer_h_
