// Simple class to contain GLSL shaders/programs

#ifndef GLSL_PROGRAM_H
#define GLSL_PROGRAM_H

#include <GL/glew.h>
#include <stdio.h>
#include "../linmath/linmath.h"

class GLSLProgramObject
{
public:
    // construct program from strings
	GLSLProgramObject() {}; 
	GLSLProgramObject(const char *vsource, const char *fsource);
    GLSLProgramObject(const char *vsource, const char *gsource, const char *fsource,
                GLenum gsInput = GL_POINTS, GLenum gsOutput = GL_TRIANGLE_STRIP);
	~GLSLProgramObject();

	void destroy(); 

	void enable();
	void disable();

	/*
	void setUniform1f(const GLchar *name, GLfloat x);
	void setUniform1i(const GLchar *name, GLint x);
	void setUniform2f(const GLchar *name, GLfloat x, GLfloat y);
    void setUniform3f(const char *name, float x, float y, float z);
    void setUniform4f(const char *name, float x, float y, float z, float w);
	*/
	//void setUniformfv(const GLchar *name, GLfloat *v, int elementSize, int count=1);
    //void setUniformMatrix4fv(const GLchar *name, GLfloat *m, bool transpose);

	int	 getLocation(const char *name); 
	// Set one uniform
	void setUniform(const char *name, const float &value); 
	void setUniform(const char *name, const chag::float2 &value); 
	void setUniform(const char *name, const chag::float3 &value); 
	void setUniform(const char *name, const chag::float4 &value); 
	void setUniform(const char *name, const int &value); 
	void setUniform(const char *name, const chag::int2 &value); 
	void setUniform(const char *name, const chag::int3 &value); 
	void setUniform(const char *name, const chag::int4 &value); 
	void setUniform(const char *name, const unsigned int &value); 
	void setUniform(const char *name, const chag::float4x4 &value); 
	void setUniform(const char *name, const chag::float3x3 &value); 
	// Set uniform array
	void setUniform(const char *name, const float *value, int count); 
	void setUniform(const char *name, const chag::float2 *value, int count); 
	void setUniform(const char *name, const chag::float3 *value, int count); 
	void setUniform(const char *name, const chag::float4 *value, int count); 
	void setUniform(const char *name, const int *value, int count); 
	void setUniform(const char *name, const chag::int2 *value, int count); 
	void setUniform(const char *name, const chag::int3 *value, int count); 
	void setUniform(const char *name, const chag::int4 *value, int count); 
	void setUniform(const char *name, const unsigned int *value, int count); 
	void setUniform(const char *name, const chag::float4x4 *value, int count); 

	void bindTexture(const char *name, GLuint tex, GLenum target, GLint unit);

	inline GLuint getProgId() { return mProg; }
	GLuint compileShader(const char *source, GLenum shaderType);
    GLuint compileProgram(const char *vsource, const char *gsource, const char *fsource,
                          GLenum gsInput = GL_POINTS, GLenum gsOutput = GL_TRIANGLE_STRIP,
						  unsigned int verticesOut = 3);
private:
	GLuint mProg;
};

#endif