#include "MAXScriptRenderer.h"
#include <iostream>
#include <fstream>
#include "Log.h"

using namespace std; 

namespace chag 
{

	MAXScriptRenderer::MAXScriptRenderer(void)
	{
	}

	MAXScriptRenderer::~MAXScriptRenderer(void)
	{
	}

	MAXScriptRenderer& MAXScriptRenderer::instance()
	{
		static MAXScriptRenderer inst;
		return inst;
	}

	void MAXScriptRenderer::flushLayer(const std::string &layerId)
	{
		ofstream os("MAXScriptRenderer.ms");
		if(os.is_open()) {
			vector<float4> vertices; 
			vector<int3> faces;

			const Layer &layer = m_items[layerId];
			const std::vector<Item> &items = layer.items;

			float4x4 Transform = layer.transform; 

			for (std::vector<Item>::const_iterator it = items.begin(); it != items.end(); ++it)
			{
				const Item &item = (*it);
				//glColor(item.color);
				switch(item.kind)
				{
				case Item::K_Cone:
				case Item::K_Sphere:
				case Item::K_Line: 
				case Item::K_Triangle: 
					LOG_WARNING("MAXScriptRenderer does not support this type");
					break;
				case Item::K_Aabb:
					{
						const Aabb &aabb = item.aabb;
						chag::float3 mi = aabb.min;
						chag::float3 ma = aabb.max;
						int base = vertices.size(); 
						vertices.push_back(Transform * make_vector(mi.x, mi.y, mi.z, 1.0f));
						vertices.push_back(Transform * make_vector(ma.x, mi.y, mi.z, 1.0f));
						vertices.push_back(Transform * make_vector(ma.x, ma.y, mi.z, 1.0f));
						vertices.push_back(Transform * make_vector(mi.x, ma.y, mi.z, 1.0f));
						vertices.push_back(Transform * make_vector(mi.x, mi.y, ma.z, 1.0f));
						vertices.push_back(Transform * make_vector(ma.x, mi.y, ma.z, 1.0f));
						vertices.push_back(Transform * make_vector(ma.x, ma.y, ma.z, 1.0f));
						vertices.push_back(Transform * make_vector(mi.x, ma.y, ma.z, 1.0f));
						faces.push_back(make_vector(base + 0, base + 1, base + 2));
						faces.push_back(make_vector(base + 2, base + 3, base + 0));
						faces.push_back(make_vector(base + 1, base + 5, base + 6));
						faces.push_back(make_vector(base + 6, base + 2, base + 1));
						faces.push_back(make_vector(base + 5, base + 4, base + 7));
						faces.push_back(make_vector(base + 7, base + 6, base + 5));
						faces.push_back(make_vector(base + 4, base + 0, base + 3));
						faces.push_back(make_vector(base + 3, base + 7, base + 4));
						faces.push_back(make_vector(base + 2, base + 6, base + 7));
						faces.push_back(make_vector(base + 7, base + 3, base + 2));
						faces.push_back(make_vector(base + 0, base + 4, base + 5));
						faces.push_back(make_vector(base + 5, base + 1, base + 0));
					};
					break;
				};
			}

			os << "vert_array = #()\n";
			os << "face_array = #()\n";
			os << "(\n";

			for(unsigned int i=0; i<vertices.size(); i++) {
				vertices[i] = (1.0f/vertices[i].w) * vertices[i]; 
				os << "append vert_array [" << vertices[i].x << "," << vertices[i].y << "," << vertices[i].z << "]\n";
			}
			for(unsigned int i=0; i<faces.size(); i++) {
				os << "append face_array [" << faces[i].x+1 << "," << faces[i].y+1 << "," << faces[i].z+1 << "]\n";
			}

			os << ")\n";

			os << "m = mesh vertices:vert_array faces:face_Array\n";
		}
		else {
			LOG_ERROR("Failed to open MAXScriptRenderer.ms"); 
		}
	}

}
