#include "orientation.h"
#include <linmath/streamoperators.h>
#include <iostream>

using namespace std; 

namespace chag
{

Orientation::Orientation(float3 pos, float3 dir, float3 up)
{
  lookAt(pos, dir, up);
}

void Orientation::lookAt(float3 eye, float3 center, float3 up)
{
  pos = eye;
  float3 dir = chag::normalize(eye - center);
  float3 right = chag::normalize(cross(up, chag::normalize(dir)));
  float3 newup = chag::normalize(cross(dir, right));
  R = make_matrix(right, newup, dir);
}

void Orientation::lookAt(float3 eye, float3 center)
{
  this->pos = eye;
  float3 dir = chag::normalize(eye - center);
  float3 right = chag::normalize(perp(dir)); 
  float3 newup = chag::normalize(cross(dir,right));
  this->R = make_matrix(right, newup, dir);
}

void Orientation::set_as_modelview()
{
	float4x4 MV = get_MV(); 
	glMatrixLoadfEXT(GL_MODELVIEW, &MV.c1.x);
}

float4x4 Orientation::get_MV()
{
	float4x4 invrot = make_matrix(transpose(R), make_vector(0.0f,0.0f,0.0f));
	return invrot * make_translation(-pos); 
}

float4x4 Orientation::get_MV_inv()
{
	return make_translation(pos) * make_matrix(R, make_vector(0.0f,0.0f,0.0f));
}

void Orientation::roll(float angle)
{
	R = R * chag::make_rotation_z<float3x3>(angle);
}
void Orientation::yaw(float angle)
{
	R = R * chag::make_rotation_y<float3x3>(angle); 
}
void Orientation::pitch(float angle)
{
	R = R * chag::make_rotation_x<float3x3>(angle); 
}

ostream & operator << ( ostream &o, const orientation &myView)
{ 
	o << myView.R << " " << myView.pos; 
	return o; 
} 

istream & operator >> ( istream &i, orientation& myView)
{
	i >> myView.R >> myView.pos; 
	return i;
}


} // namespace chag
